# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

from .annotation_param import AnnotationParam

__all__ = ["TextContentParam"]


class TextContentParam(TypedDict, total=False):
    """A text content block."""

    text: Required[str]
    """Required. The text content."""

    type: Required[Literal["text"]]

    annotations: Iterable[AnnotationParam]
    """Citation information for model-generated content."""
