# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .google_maps_result import GoogleMapsResult

__all__ = ["GoogleMapsResultContent"]


class GoogleMapsResultContent(BaseModel):
    """Google Maps result content."""

    call_id: str
    """Required. ID to match the ID from the function call block."""

    result: List[GoogleMapsResult]
    """Required. The results of the Google Maps."""

    type: Literal["google_maps_result"]

    signature: Optional[str] = None
    """A signature hash for backend validation."""
